/* Copyright (c) 2006 by Michael J. Roberts.  All Rights Reserved. */
/*
Name
  tadscom.h - COM helper routines
Function
  
Notes
  
Modified
  10/09/06 MJRoberts  - Creation
*/

#ifndef TADSCOM_H
#define TADSCOM_H

#include <Windows.h>
#include <wtypes.h>

#include <stdlib.h>
#include <stddef.h>
#include <string.h>


/* 
 *   Allocate a BSTR from an ANSI string.
 */
BSTR bstr_from_ansi(const textchar_t *str, size_t len);

/*
 *   Allocate a BSTR from a null-terminated ANSI string.
 */
BSTR bstr_from_ansi(const textchar_t *str);

/*
 *   Convert a BSTR into an ANSI string buffer, copying the ANSI characters
 *   into a buffer.  
 */
textchar_t *ansi_from_bstr(
    textchar_t *dst, size_t dstsize, BSTR bstr, int free_bstr);

/*
 *   Allocate an ANSI string buffer from a BSTR 
 */
textchar_t *ansi_from_bstr(BSTR bstr, int free_bstr);

/*
 *   Convert an OLESTR string to an ANSI string 
 */
textchar_t *ansi_from_olestr(
    textchar_t *dst, size_t dstsize, const OLECHAR *olestr, size_t olecnt);
inline textchar_t *ansi_from_olestr(
    textchar_t *dst, size_t dstsize, const OLECHAR *olestr)
{
    return ansi_from_olestr(dst, dstsize, olestr, wcslen(olestr));
}

/*
 *   Allocate an ANSI string buffer from an OLESTR 
 */
inline textchar_t *ansi_from_olestr(const OLECHAR *olestr, size_t olecnt);
inline textchar_t *ansi_from_olestr(const OLECHAR *olestr)
{
    return ansi_from_olestr(olestr, wcslen(olestr));
}

#endif /* TADSCOM_H */
