/* Copyright (c) 2006 by Michael J. Roberts.  All Rights Reserved. */
/*
Name
  w32help.cpp - Workbench help/doc viewer tool window
Function
  
Notes
  
Modified
  12/29/06 MJRoberts  - Creation
*/

#include <Windows.h>
#include <exdisp.h>
#include <oleauto.h>
#include <exdispid.h>
#include <mshtml.h>
#include <mshtmhst.h>
#include <MsHtmcid.h>

#include "tadshtml.h"
#include "tadsapp.h"
#include "w32tdb.h"
#include "w32help.h"
#include "htmlw32.h"
#include "htmldbg.h"
#include "htmldcfg.h"
#include "tadscom.h"


/* ------------------------------------------------------------------------ */
/*
 *   Our base OLE object class.  This manages the OLE reference counting. 
 */
class CBaseOle
{
public:
    CBaseOle()
    {
        /* 
         *   construction automatically gives us one initial reference, from
         *   the caller who constructed us 
         */
        refcnt_ = 1;
    }
    
    virtual ~CBaseOle() { }

protected:
    ULONG refcnt_;
};

/*
 *   Define the base IUnknown methods for an object.  This defines the
 *   AddRef/ReleaseRef methods using the reference count from the CBaseOle
 *   base class.  
 */
#define BaseIUnknown \
    ULONG STDMETHODCALLTYPE AddRef() { return ++refcnt_; } \
    ULONG STDMETHODCALLTYPE Release() \
    { \
        ULONG ret = --refcnt_; \
        if (ret == 0) \
            delete this; \
        return ret; \
    }

/* ------------------------------------------------------------------------ */
/*
 *   Our IStorage interface.  The browser object doesn't use this object;
 *   it's just required for the standard initialization protocol for any
 *   in-place object.  So we merely need to define most of these methods as
 *   'not implemented'.  
 */
class CDummyStorage: public CBaseOle, public IStorage
{
public:
    BaseIUnknown;

    HRESULT STDMETHODCALLTYPE QueryInterface(REFIID iid, LPVOID FAR *obj)
    {
        *obj = 0;
        return E_NOTIMPL;
    }

    HRESULT STDMETHODCALLTYPE CreateStream(
        const WCHAR *pwcsName, DWORD grfMode,
        DWORD reserved1, DWORD reserved2, IStream **ppstm)
        { return E_NOTIMPL; }

    HRESULT STDMETHODCALLTYPE OpenStream(
        const WCHAR * pwcsName, void *reserved1, DWORD grfMode,
        DWORD reserved2, IStream **ppstm)
        { return E_NOTIMPL; }

    HRESULT STDMETHODCALLTYPE CreateStorage(
        const WCHAR *pwcsName, DWORD grfMode,
        DWORD reserved1, DWORD reserved2, IStorage **ppstg)
        { return E_NOTIMPL; }

    HRESULT STDMETHODCALLTYPE OpenStorage(
        const WCHAR *pwcsName, IStorage *pstgPriority, DWORD grfMode,
        SNB snbExclude, DWORD reserved, IStorage **ppstg)
        { return E_NOTIMPL; }

    HRESULT STDMETHODCALLTYPE CopyTo(
        DWORD ciidExclude, IID const *rgiidExclude, SNB snbExclude,
        IStorage *pstgDest)
        { return E_NOTIMPL; }

    HRESULT STDMETHODCALLTYPE MoveElementTo(
        const OLECHAR *pwcsName, IStorage *pstgDest,
        const OLECHAR *pwcsNewName, DWORD grfFlags)
        { return E_NOTIMPL; }

    HRESULT STDMETHODCALLTYPE Commit(DWORD grfCommitFlags)
        { return E_NOTIMPL; }

    HRESULT STDMETHODCALLTYPE Revert()
        { return E_NOTIMPL; }

    HRESULT STDMETHODCALLTYPE EnumElements(
        DWORD reserved1, void *reserved2, DWORD reserved3,
        IEnumSTATSTG **ppenum)
        { return E_NOTIMPL; }

    HRESULT STDMETHODCALLTYPE DestroyElement(
        const OLECHAR *pwcsName)
        { return E_NOTIMPL; }

    HRESULT STDMETHODCALLTYPE RenameElement(
        const WCHAR *pwcsOldName, const WCHAR *pwcsNewName)
        { return E_NOTIMPL; }

    HRESULT STDMETHODCALLTYPE SetElementTimes(
        const WCHAR *pwcsName, FILETIME const *pctime,
        FILETIME const *patime, FILETIME const *pmtime)
        { return E_NOTIMPL; }

    HRESULT STDMETHODCALLTYPE SetClass(REFCLSID clsid)
        { return S_OK; }

    HRESULT STDMETHODCALLTYPE SetStateBits(DWORD grfStateBits, DWORD grfMask)
        { return E_NOTIMPL; }

    HRESULT STDMETHODCALLTYPE Stat(STATSTG * pstatstg, DWORD grfStatFlag)
        { return E_NOTIMPL; }
};

/* ------------------------------------------------------------------------ */
/*
 *   Web browser event sink - this receives events that the html control
 *   sends out to registered DWebBrowserEvents2 objects 
 */
class CWebEventSink: public CBaseOle, public IDispatch
{
public:
    CWebEventSink(CHtmlSys_dbghelpwin *win, IWebBrowser2 *iwb2)
    {
        /* remember my containing window */
        win_ = win;

        /* remember the web browser control interface */
        iwb2->AddRef();
        iwb2_ = iwb2;

        /* we're not yet registered */
        registered_ = FALSE;

        /* register */
        register_event_handler(TRUE);
    }

    ~CWebEventSink()
    {
        /* make sure we're unregistered */
        register_event_handler(FALSE);

        /* release our web browser interface */
        iwb2_->Release();
    }

    BaseIUnknown;
    HRESULT STDMETHODCALLTYPE QueryInterface(REFIID iid, LPVOID FAR *obj)
    {
        /* we're an IDispatch, and of course also IUnknown */
        if (iid == IID_IUnknown || iid == IID_IDispatch)
        {
            *obj = (IDispatch *)this;
            AddRef();
            return S_OK;
        }

        /* we don't provide any other interfaces */
        return E_NOINTERFACE;
    }

    HRESULT STDMETHODCALLTYPE GetTypeInfoCount(unsigned int FAR *cnt)
    {
        *cnt = 0;
        return S_OK;
    }

    HRESULT STDMETHODCALLTYPE GetTypeInfo(
        unsigned int itinfo, LCID lcid, ITypeInfo FAR *FAR *info)
        { return E_NOTIMPL; }

    HRESULT STDMETHODCALLTYPE GetIDsOfNames(
        REFIID riid, OLECHAR FAR *FAR *rgszNames, UINT cNames,
        LCID lcid, DISPID FAR *rgDispId)
        { return E_NOTIMPL; }

    HRESULT STDMETHODCALLTYPE Invoke( 
        DISPID member, REFIID riid, LCID lcid, WORD flags,
        DISPPARAMS FAR *params, VARIANT FAR *result,
        EXCEPINFO FAR *exc, unsigned int FAR *arg_err)
    {
        /* check which member we've invoking */
        switch(member)
        {
        case DISPID_NEWWINDOW2:
            /* 
             *   trying to open a new window - always force the navigation to
             *   occur in the current window 
             */
            {
                IWebBrowser2 *new_iwb2;
                IDispatch *idisp;
                
                /* create a new embedded browser window */
                new_iwb2 = win_->new_browser_ctl();

                /* if that succeeded, use its IDispatch as the new target */
                if (new_iwb2 != 0
                    && SUCCEEDED(new_iwb2->QueryInterface(
                        IID_IDispatch, (void **)&idisp)))
                {
                    VARIANT v = params->rgvarg[1];
                    *v.ppdispVal = idisp;
                }
            }
            break;
            
        case DISPID_QUIT:
            /* disconnect */
            register_event_handler(FALSE);
            break;

        case DISPID_NAVIGATECOMPLETE2:
            /* navigating to a new URL */
            {
                /* get the parameters: the frame IDispatch and the URL */
                BSTR url = params->rgvarg[0].pvarVal->bstrVal;
                IDispatch *framedisp = params->rgvarg[1].pdispVal;

                /* only pay attention to changes in the main frame */
                IWebBrowser2 *frame;
                if (SUCCEEDED(framedisp->QueryInterface(
                    IID_IWebBrowser2, (void **)&frame)))
                {
                    /* if it's our main frame, note the new URL */
                    if (frame == iwb2_)
                    {
                        /* convert to ANSI */
                        textchar_t *urla = ansi_from_bstr(url, FALSE);

                        /* note the change */
                        win_->on_url_change(urla);

                        /* done with the ANSI version */
                        th_free(urla);
                    }

                    /* done with the frame */
                    frame->Release();
                }
            }
            break;

#if 0
        // we don't currently need the document title, but here's
        // how we'd get it if we did...
        case DISPID_DOCUMENTCOMPLETE:
            /* loaded a document */
            {
                BSTR url = params->rgvarg[0].pvarVal->bstrVal;
                IDispatch *framedisp = params->rgvarg[1].pdispVal;
                IWebBrowser2 *frame;

                if (SUCCEEDED(framedisp->QueryInterface(
                    IID_IWebBrowser2, (void **)&frame)))
                {
                    if (frame == iwb2_)
                    {
                        IDispatch *doc;
                        
                        if (SUCCEEDED(frame->get_Document(&doc)))
                        {
                            IHTMLDocument2 *doc2;
                            
                            if (SUCCEEDED(doc->QueryInterface(
                                IID_IHTMLDocument2, (void **)&doc2)))
                            {
                                BSTR title;
                                
                                if (SUCCEEDED(doc2->get_title(&title)))
                                {
                                    note_new_title(title);
                                    SysFreeString(title);
                                }
                                doc2->Release();
                            }
                            doc->Release();
                        }
                        
                    }

                    frame->Release();
                }
            }
            break;
#endif
        }

        /* handled */
        return S_OK;
    }

    /* register or unregister o\ur web browser event handler */
    void register_event_handler(int reg)
    {
        IConnectionPointContainer *cpcont;

        /* if we're already in the desired state, ignore the request */
        if ((reg && registered_) || (!reg && !registered_))
            return;

        /* get the connection point container */
        if (SUCCEEDED(iwb2_->QueryInterface(
            IID_IConnectionPointContainer, (void **)&cpcont)))
        {
            IConnectionPoint *cp;

            /* get the connection point for web browser events */
            if (SUCCEEDED(cpcont->FindConnectionPoint(
                DIID_DWebBrowserEvents2, &cp)))
            {
                /* register or unregister, as indicated by the caller */
                if (reg)
                    cp->Advise((IDispatch *)this, &cookie_);
                else
                    cp->Unadvise(cookie_);

                /* remember the new registration status */
                registered_ = reg;

                /* release the connection point object */
                cp->Release();
            }

            /* release the connection point container object */
            cpcont->Release();
        }
    }

protected:
    /* my containing window */
    CHtmlSys_dbghelpwin *win_;

    /* the browser control interface */
    IWebBrowser2 *iwb2_;

    /* the "cookie" our connection point uses to identify us */
    DWORD cookie_;

    /* are we registered? */
    int registered_;
};


/* ------------------------------------------------------------------------ */
/*
 *   Our in-place frame 
 */

/* the frame class */
class CFrame: public CBaseOle, public IOleInPlaceFrame
{
public:
    CFrame(HWND hwnd)
    {
        /* remember the frame handle */
        hwnd_ = hwnd;

        /* no active object yet */
        actobj_ = 0;
    }

    ~CFrame()
    {
        if (actobj_ != 0)
            actobj_->Release();
    }

    /* get the active object's window handle */
    HWND get_obj_hwnd()
    {
        HWND hwnd;
        
        return (actobj_ != 0 && SUCCEEDED(actobj_->GetWindow(&hwnd))
                ? hwnd : 0);
    }
    
    BaseIUnknown;
    HRESULT STDMETHODCALLTYPE QueryInterface(REFIID iid, LPVOID FAR *obj)
    {
        /* we don't have to expose any interfaces to embed a browser */
        *obj = 0;
        return E_NOTIMPL;
    }

    HRESULT STDMETHODCALLTYPE GetWindow(HWND FAR *lphwnd)
    {
        /* give the browser our window handle */
        *lphwnd = hwnd_;
        return S_OK;
    }

    HRESULT STDMETHODCALLTYPE ContextSensitiveHelp(BOOL fEnterMode)
        { return E_NOTIMPL; }

    HRESULT STDMETHODCALLTYPE GetBorder(LPRECT lprectBorder)
        { return E_NOTIMPL; }

    HRESULT STDMETHODCALLTYPE RequestBorderSpace(LPCBORDERWIDTHS pbw)
        { return E_NOTIMPL; }

    HRESULT STDMETHODCALLTYPE SetBorderSpace(LPCBORDERWIDTHS pborderwidths)
        { return E_NOTIMPL; }

    HRESULT STDMETHODCALLTYPE SetActiveObject(
        IOleInPlaceActiveObject *pActiveObject, LPCOLESTR pszObjName)
    {
        /* remember the active object */
        if (pActiveObject != 0)
            pActiveObject->AddRef();
        if (actobj_ != 0)
            actobj_->Release();
        actobj_ = pActiveObject;

        /* success */
        return S_OK;
    }

    HRESULT STDMETHODCALLTYPE InsertMenus(
        HMENU hmenuShared, LPOLEMENUGROUPWIDTHS lpMenuWidths)
        { return E_NOTIMPL; }

    HRESULT STDMETHODCALLTYPE SetMenu(
        HMENU hmenuShared, HOLEMENU holemenu, HWND hwndActiveObject)
        { return S_OK; }

    HRESULT STDMETHODCALLTYPE RemoveMenus(HMENU hmenuShared)
        { return E_NOTIMPL; }

    HRESULT STDMETHODCALLTYPE SetStatusText(LPCOLESTR pszStatusText)
        { return S_OK; }

    HRESULT STDMETHODCALLTYPE EnableModeless(BOOL fEnable)
        { return S_OK; }

    HRESULT STDMETHODCALLTYPE TranslateAccelerator(LPMSG lpmsg, WORD wID)
        { return E_NOTIMPL; }

protected:
    /* our container window handle */
    HWND hwnd_;

    /* our active object */
    IOleInPlaceActiveObject *actobj_;
};


/* ------------------------------------------------------------------------ */
/*
 *   Our in-place site 
 */
class CInPlaceSite: public CBaseOle, public IOleInPlaceSite
{
public:
    CInPlaceSite(HWND hwnd, IOleClientSite *site)
    {
        /* remember our window */
        hwnd_ = hwnd;

        /* 
         *   remember our parent client site (don't add a reference to it,
         *   since it owns us - adding a reference would create a circular
         *   reference that would prevent us from ever being deleted) 
         */
        client_site_ = site;

        /* create our frame object */
        frame_ = new CFrame(hwnd);

        /* 
         *   we don't have a browser yet (we can't yet, since we need to
         *   create all of our host interfaces before we can embed the
         *   browser object) 
         */
        browser_ = 0;
    }

    ~CInPlaceSite()
    {
        /* release references to the object's we're holding onto */
        if (browser_ != 0)
            browser_->Release();
        if (frame_ != 0)
            frame_->Release();
    }

    /* get the active object's window handle, if available */
    HWND get_active_obj_hwnd()
    {
        return (frame_ != 0 ? frame_->get_obj_hwnd() : 0);
    }

    /* set the browser object */
    void set_browser(IOleObject *browser)
    {
        /* if there's a new object, add a reference to it */
        if (browser != 0)
            browser->AddRef();

        /* if we already have a browser, release the old one */
        if (browser_ != 0)
            browser_->Release();

        /* remember the new one */
        browser_ = browser;
    }

    BaseIUnknown;
    HRESULT STDMETHODCALLTYPE QueryInterface(REFIID iid, LPVOID FAR *obj)
    {
        /* let our client site implementation handle it */
        return client_site_->QueryInterface(iid, obj);
    }

    HRESULT STDMETHODCALLTYPE GetWindow(HWND FAR *lphwnd)
    {
        /* return the HWND of our main window */
        *lphwnd = hwnd_;
        return S_OK;
    }

    HRESULT STDMETHODCALLTYPE ContextSensitiveHelp(BOOL fEnterMode)
        { return E_NOTIMPL; }

    HRESULT STDMETHODCALLTYPE CanInPlaceActivate()
    {
        /* tell the browser we can activate in place */
        return S_OK;
    }

    HRESULT STDMETHODCALLTYPE OnInPlaceActivate()
    {
        /* simply indicate success */
        return S_OK;
    }

    HRESULT STDMETHODCALLTYPE OnUIActivate()
    {
        /* simply indicate success */
        return S_OK;
    }

    HRESULT STDMETHODCALLTYPE GetWindowContext(
        LPOLEINPLACEFRAME FAR *lplpFrame, LPOLEINPLACEUIWINDOW FAR *lplpDoc,
        LPRECT lprcPosRect, LPRECT lprcClipRect,
        LPOLEINPLACEFRAMEINFO lpFrameInfo)
    {
        /* 
         *   pass back our in-place frame implementation; add a reference to
         *   it on behalf of the caller 
         */
        frame_->AddRef();
        *lplpFrame = frame_;

        /* We have no OLEINPLACEUIWINDOW */
        *lplpDoc = 0;

        /* we're not MDI */
        lpFrameInfo->fMDIApp = FALSE;

        /* return our frame window */
        lpFrameInfo->hwndFrame = hwnd_;

        /* we have no accelerators of our own */
        lpFrameInfo->haccel = 0;
        lpFrameInfo->cAccelEntries = 0;

        /* give the browser the frame window dimensions */
        GetClientRect(hwnd_, lprcPosRect);
        GetClientRect(hwnd_, lprcClipRect);

        /* success */
        return S_OK;
    }

    HRESULT STDMETHODCALLTYPE Scroll(SIZE scrollExtent)
        { return E_NOTIMPL; }

    HRESULT STDMETHODCALLTYPE OnUIDeactivate(BOOL fUndoable)
    {
        /* simply indicate success */
        return S_OK;
    }

    HRESULT STDMETHODCALLTYPE OnInPlaceDeactivate()
    {
        /* simply indicate success */
        return S_OK;
    }

    HRESULT STDMETHODCALLTYPE DiscardUndoState()
        { return E_NOTIMPL; }

    HRESULT STDMETHODCALLTYPE DeactivateAndUndo()
        { return E_NOTIMPL; }

    HRESULT STDMETHODCALLTYPE OnPosRectChange(LPCRECT lprcPosRect)
    {
        IOleInPlaceObject *inplace;

        /* get the browser's in-place object */
        if (browser_ != 0
            && SUCCEEDED(browser_->QueryInterface(
            IID_IOleInPlaceObject, (void**)&inplace)))
        {
            /* give the browser the dimensions of where it can draw */
            inplace->SetObjectRects(lprcPosRect, lprcPosRect);
        }

        /* success */
        return S_OK;
    }

protected:
    /* the window containing the site */
    HWND hwnd_;

    /* our client site implementation */
    IOleClientSite *client_site_;

    /* our in-place frame implementation */
    CFrame *frame_;

    /* the browser object we're embedding */
    IOleObject *browser_;
};

/* ------------------------------------------------------------------------ */
/*
 *   Our document host 
 */
class CDocHost: public CBaseOle, public IDocHostUIHandler
{
public:
    CDocHost(HWND hwnd, IOleClientSite *site)
    {
        /* remember the client site - it owns us, so don't add a reference */
        client_site_ = site;

        /* remember the window */
        hwnd_ = hwnd;
    }

    BaseIUnknown;
    HRESULT STDMETHODCALLTYPE QueryInterface(REFIID iid, LPVOID FAR *obj)
    {
        /* let our associated IOleClientSite object handle it */
        return client_site_->QueryInterface(iid, obj);
    }

    HRESULT STDMETHODCALLTYPE ShowContextMenu(
        DWORD dwID, POINT __RPC_FAR *ppt, IUnknown __RPC_FAR *pcmdtReserved,
        IDispatch __RPC_FAR *pdispReserved)
    {
        HMENU m;
        
        /* show our context menu */
        m = LoadMenu(CTadsApp::get_app()->get_instance(),
                     MAKEINTRESOURCE(IDR_HELP_POPUP_MENU));

        /* run the menu */
        TrackPopupMenu(GetSubMenu(m, 0), TPM_TOPALIGN | TPM_LEFTALIGN,
                       ppt->x, ppt->y, 0, hwnd_, 0);

        /* done with the menu */
        DestroyMenu(m);

        /* indicate success */
        return S_OK;
    }

    HRESULT STDMETHODCALLTYPE GetHostInfo(DOCHOSTUIINFO __RPC_FAR *pInfo)
    {
        /* set the structure size, to indicate interface compatibility */
        pInfo->cbSize = sizeof(DOCHOSTUIINFO);

        /* we don't want a 3D border */
        pInfo->dwFlags = DOCHOSTUIFLAG_NO3DBORDER;

        /* use default double-clicking behavior */
        pInfo->dwDoubleClick = DOCHOSTUIDBLCLK_DEFAULT;

        /* success */
        return S_OK;
    }

    HRESULT STDMETHODCALLTYPE ShowUI(
        DWORD dwID, IOleInPlaceActiveObject __RPC_FAR *pActiveObject,
        IOleCommandTarget __RPC_FAR *pCommandTarget,
        IOleInPlaceFrame __RPC_FAR *pFrame,
        IOleInPlaceUIWindow __RPC_FAR *pDoc)
    {
        /* we already have our UI in place, so there's nothing more to do */
        return S_OK;
    }

    HRESULT STDMETHODCALLTYPE HideUI()
    {
        /* simply indicate success */
        return S_OK;
    }

    HRESULT STDMETHODCALLTYPE UpdateUI()
    {
        /* simply indicate success */
        return S_OK;
    }

    HRESULT STDMETHODCALLTYPE EnableModeless(BOOL fEnable)
    {
        /* simply indicate success */
        return S_OK;
    }

    HRESULT STDMETHODCALLTYPE OnDocWindowActivate(BOOL fActivate)
    {
        /* simply indicate success */
        return S_OK;
    }

    HRESULT STDMETHODCALLTYPE OnFrameWindowActivate(BOOL fActivate)
    {
        /* simply indicate success */
        return S_OK;
    }

    HRESULT STDMETHODCALLTYPE ResizeBorder(
        LPCRECT prcBorder, IOleInPlaceUIWindow __RPC_FAR *pUIWindow,
        BOOL fRameWindow)
    {
        /* simply indicate success */
        return S_OK;
    }

    HRESULT STDMETHODCALLTYPE TranslateAccelerator(
        LPMSG lpMsg, const GUID __RPC_FAR *pguidCmdGroup, DWORD nCmdID)
    {
        /* we don't translate any accelerators */
        return S_FALSE;
    }

    HRESULT STDMETHODCALLTYPE GetOptionKeyPath(
        LPOLESTR __RPC_FAR *pchKey, DWORD dw)
    {
        /* let the browser use its default registry keys */
        return S_FALSE;
    }

    HRESULT STDMETHODCALLTYPE GetDropTarget(
        IDropTarget __RPC_FAR *pDropTarget,
        IDropTarget __RPC_FAR *__RPC_FAR *ppDropTarget)
    {
        /* we don't need a drop target */
        return S_FALSE;
    }

    HRESULT STDMETHODCALLTYPE GetExternal(
        IDispatch __RPC_FAR *__RPC_FAR *ppDispatch)
    {
        /* we don't provide an IDispatch object */
        *ppDispatch = 0;
        return S_FALSE;
    }

    HRESULT STDMETHODCALLTYPE TranslateUrl(
        DWORD dwTranslate, OLECHAR __RPC_FAR *pchURLIn,
        OLECHAR __RPC_FAR *__RPC_FAR *ppchURLOut)
    {
        /* we don't need to do any URL translations */
        *ppchURLOut = 0;
        return S_FALSE;
    }

    HRESULT STDMETHODCALLTYPE FilterDataObject(
        IDataObject __RPC_FAR *pDO, IDataObject __RPC_FAR *__RPC_FAR *ppDORet)
    {
        /* we don't provide an IDataObject */
        *ppDORet = 0;
        return(S_FALSE);
    }

protected:
    /* our associated IOleClientSite object */
    IOleClientSite *client_site_;

    /* our window handle */
    HWND hwnd_;
};


/* ------------------------------------------------------------------------ */
/*
 *   Our client site 
 */
class CClientSite: public CBaseOle, public IOleClientSite
{
public:
    CClientSite(HWND hwnd)
    {
        /* create our in-place site */
        in_place_site_ = new CInPlaceSite(hwnd, this);

        /* create our document host */
        doc_host_ = new CDocHost(hwnd, this);
    }

    ~CClientSite()
    {
        /* release references on our owned objects */
        if (in_place_site_ != 0)
            in_place_site_->Release();
        if (doc_host_ != 0)
            doc_host_->Release();
    }

    /* get the active object's window handle, if available */
    HWND get_active_obj_hwnd()
    {
        return (in_place_site_ != 0
                ? in_place_site_->get_active_obj_hwnd() : 0);
    }

    /* set the browser */
    void set_browser(IOleObject *browser)
    {
        /* 
         *   we don't need to know about the browser, but our in-place site
         *   does, so pass it through 
         */
        in_place_site_->set_browser(browser);
    }
    
    BaseIUnknown;
    HRESULT STDMETHODCALLTYPE QueryInterface(REFIID iid, LPVOID FAR *obj)
    {
        /* check for IUnknown and IOleClientSite */
        if (iid == IID_IUnknown || iid == IID_IOleClientSite)
        {
            /* return 'this', adding a reference on behalf of the caller */
            *obj = (IOleClientSite *)this;
            AddRef();
            return S_OK;
        }

        /* check for IOleInPlaceSite */
        if (iid == IID_IOleInPlaceSite)
        {
            /* return our in-place site object, adding a reference */
            in_place_site_->AddRef();
            *obj = (IOleInPlaceSite *)in_place_site_;
            return S_OK;
        }

        /* check for IDocHostUIHandler */
        if (iid == IID_IDocHostUIHandler)
        {
            /* return our document host object, adding a reference */
            doc_host_->AddRef();
            *obj = (IDocHostUIHandler *)doc_host_;
            return S_OK;
        }

        /* it's not one we recognize */
        *obj = 0;
        return E_NOINTERFACE;
    }

    HRESULT STDMETHODCALLTYPE SaveObject()
        { return E_NOTIMPL; }

    HRESULT STDMETHODCALLTYPE GetMoniker(
        DWORD dwAssign, DWORD dwWhichMoniker, IMoniker **ppmk)
        { return E_NOTIMPL; }

    HRESULT STDMETHODCALLTYPE GetContainer(LPOLECONTAINER FAR *ppContainer)
    {
        /* report that we don't support a container */
        *ppContainer = 0;
        return E_NOINTERFACE;
    }

    HRESULT STDMETHODCALLTYPE ShowObject()
        { return NOERROR; }

    HRESULT STDMETHODCALLTYPE OnShowWindow(BOOL fShow)
        { return E_NOTIMPL; }

    HRESULT STDMETHODCALLTYPE RequestNewObjectLayout()
        { return E_NOTIMPL; }

protected:
    /* our in-place site object */
    CInPlaceSite *in_place_site_;

    /* our document host UI handler */
    IDocHostUIHandler *doc_host_;
};




/* ------------------------------------------------------------------------ */
/*
 *   The CHtmlSys_dbghelpwin window class.  This is the Workbench window
 *   object; it's a dockable window, so the actual system window lives inside
 *   another system window (which is an MDI child or a docking window,
 *   depending on the current docking state).  
 */

/*
 *   construction 
 */
CHtmlSys_dbghelpwin::CHtmlSys_dbghelpwin(
    class CHtmlPreferences *prefs,
    class CHtmlDebugHelper *helper,
    class CHtmlSys_dbgmain *dbgmain)
{
    /* remember our prefs, helper, and main window objects */
    prefs_ = prefs;
    helper_ = helper;
    dbgmain_ = dbgmain;
    dbgmain_->AddRef();

    /* we don't have a browser or an embedding site yet */
    browser_ = 0;
    iwb2_ = 0;
    icmd_ = 0;
    storage_ = 0;
    client_site_ = 0;
    web_evt_ = 0;
}

/*
 *   deletion 
 */
CHtmlSys_dbghelpwin::~CHtmlSys_dbghelpwin()
{
}

/*
 *   replace the embedded web browser control with a new instance 
 */
IWebBrowser2 *CHtmlSys_dbghelpwin::new_browser_ctl()
{
    /* embed a new browser */
    embed_browser();

    /* return the new control */
    return iwb2_;
}

/*
 *   system window creation 
 */
void CHtmlSys_dbghelpwin::do_create()
{
    /* inherit the default handling */
    CTadsWin::do_create();

    /* create our embedded browser control */
    embed_browser();

    /* set up a timer to watch for focus changes */
    focus_timer_ = alloc_timer_id();
    SetTimer(handle_, focus_timer_, 500, 0);

    /* assume we don't have focus */
    has_focus_ = FALSE;
}

/*
 *   system window destruction
 */
void CHtmlSys_dbghelpwin::do_destroy()
{
    /* delete our browser control */
    unembed_browser();

    /* notify the debugger interface */
    dbgmain_->dbghostifc_on_destroy(this, this);

    /* tell the debugger we're closing an active window */
    dbgmain_->on_close_dbg_win(this);

    /* release the interface */
    dbgmain_->Release();

    /* delete our timer */
    KillTimer(handle_, focus_timer_);
    free_timer_id(focus_timer_);

    /* do the inherited work */
    CTadsWin::do_destroy();
}

/* 
 *   embed a browser object in our window 
 */
int CHtmlSys_dbghelpwin::embed_browser()
{
    RECT rc;
    
    /* if we already have an embedded browser, remove it */
    unembed_browser();
    
    /* create our client site */
    client_site_ = new CClientSite(handle_);
    storage_ = new CDummyStorage();
    
    /* create the WebBrowser control */
    if (!OleCreate(CLSID_WebBrowser, IID_IOleObject, OLERENDER_DRAW,
                   0, client_site_, storage_, (void**)&browser_)
        && browser_ != 0)
    {
        /* tell the client site about the browser */
        client_site_->set_browser(browser_);
        
        /* set the host name */
        browser_->SetHostNames(L"TADS Workbench", 0);
        
        /* get our window client area */
        GetClientRect(handle_, &rc);
        
        /* 
         *   Set up the OLE container embedding, show the browser control,
         *   and ask for its IWebBrowser2 interface 
         */
        if (!OleSetContainedObject(browser_, TRUE)
            && SUCCEEDED(browser_->DoVerb(OLEIVERB_SHOW, NULL,
                                          client_site_, -1, handle_, &rc))
            && SUCCEEDED(browser_->QueryInterface(
                IID_IWebBrowser2, (void**)&iwb2_)))
        {
            /* set the display area to take up our whole client area */
            iwb2_->put_Left(0);
            iwb2_->put_Top(0);
            iwb2_->put_Width(rc.right);
            iwb2_->put_Height(rc.bottom);
            
            /* get the IOleCommandTarget interface */
            browser_->QueryInterface(IID_IOleCommandTarget, (void **)&icmd_);

            /* set up our event handler */
            web_evt_ = new CWebEventSink(this, iwb2_);
        }
        
        /* success */
        return TRUE;
    }
    
    /* clean up anything we successfully created */
    unembed_browser();
    
    /* return failure */
    return FALSE;
}

/* 
 *   un-embed a browser object 
 */
void CHtmlSys_dbghelpwin::unembed_browser()
{
    /* if we have a browser, remvoe it */
    if (browser_ != 0)
    {
        /* close the browser */
        browser_->Close(OLECLOSE_NOSAVE);
        
        /* detach the browser from our client site */
        browser_->SetClientSite(0);
        
        /* detach the browser from the site */
        client_site_->set_browser(0);
        
        /* release and forget the browser object */
        browser_->Release();
        browser_ = 0;
    }
    
    /* unregister and release our event sink */
    if (web_evt_ != 0)
    {
        web_evt_->register_event_handler(FALSE);
        web_evt_->Release();
        web_evt_ = 0;
    }

    /* delete our host interface implementation objects */
    if (iwb2_ != 0)
    {
        iwb2_->Release();
        iwb2_ = 0;
    }
    if (icmd_ != 0)
    {
        icmd_->Release();
        icmd_ = 0;
    }
    if (storage_ != 0)
    {
        storage_->Release();
        storage_ = 0;
    }
    if (client_site_ != 0)
    {
        client_site_->Release();
        client_site_ = 0;
    }
}

/*
 *   check command status 
 */
TadsCmdStat_t CHtmlSys_dbghelpwin::check_command(const check_cmd_info *info)
{
    /* route it to the main window */
    return dbgmain_->check_command(info);
}

/*
 *   execute a command
 */
int CHtmlSys_dbghelpwin::do_command(int notify_code, int command_id, HWND ctl)
{
    /* route it to the main window */
    return dbgmain_->do_command(notify_code, command_id, ctl);
}

/*
 *   translate a Workbench ID_xxx command to an OLE command ID 
 */
static DWORD get_olecmd(int command_id, OLECMDEXECOPT *optp)
{
    OLECMDEXECOPT opt = OLECMDEXECOPT_DONTPROMPTUSER;
    DWORD oleid = IDM_UNKNOWN;
    
    switch (command_id)
    {
    case ID_HELP_GOBACK:
        oleid = IDM_GOBACKWARD;
        break;

    case ID_HELP_GOFORWARD:
        oleid = IDM_GOFORWARD;
        break;

    case ID_HELP_REFRESH:
        oleid = IDM_REFRESH;
        break;

    case ID_EDIT_COPY:
        oleid = IDM_COPY;
        break;

    case ID_EDIT_SELECTALL:
        oleid = IDM_SELECTALL;
        break;

    case ID_EDIT_FIND:
        oleid = IDM_FIND;
        opt = OLECMDEXECOPT_PROMPTUSER;
        break;

    case ID_FILE_PRINT:
        oleid = IDM_PRINT;
        opt = OLECMDEXECOPT_PROMPTUSER;
        break;
    }

    /* return the options, if desired */
    if (optp != 0)
        *optp = opt;

    /* return the OLE command ID */
    return oleid;
}

/*
 *   check command status 
 */
TadsCmdStat_t CHtmlSys_dbghelpwin::active_check_command(
    const check_cmd_info *info)
{
    OLECMD olecmd;

    /* check for our own commands before any OLE translation */
    switch (info->command_id)
    {
    case ID_EDIT_FIND:
        /* update the menu to remove any past filename indication */
        check_command_change_menu(info, IDS_TPL_FIND_BLANK);
        break;

    case ID_FILE_PAGE_SETUP:
        /* 
         *   don't allow this - we don't have a way to pass the page setup
         *   information to the browser control, so make it clear that our
         *   page setup settings don't apply to the Help/Doc window by
         *   disabling the dialog entirely 
         */
        return TADSCMD_DISABLED;
    }

    /* check for a translation to an OLE command */
    if (icmd_ != 0
        && (olecmd.cmdID = get_olecmd(info->command_id, 0)) != IDM_UNKNOWN
        && SUCCEEDED(icmd_->QueryStatus(&CGID_MSHTML, 1, &olecmd, 0)))
    {
        /* return the OLE command status */
        if ((olecmd.cmdf & OLECMDF_LATCHED) != 0)
            return ((olecmd.cmdf & OLECMDF_ENABLED) != 0
                    ? TADSCMD_CHECKED : TADSCMD_DISABLED_CHECKED);
        else
            return ((olecmd.cmdf & OLECMDF_ENABLED) != 0
                    ? TADSCMD_ENABLED : TADSCMD_DISABLED);
    }

    /* inherit base class behavior for other commands */
    return CTadsWin::check_command(info);
}

int CHtmlSys_dbghelpwin::active_do_command(
    int notify_code, int command_id, HWND ctl)
{
    DWORD olecmd;
    OLECMDEXECOPT oleopt;

    /* check for translations to OLECMDID codes */
    if (icmd_ != 0
        && (olecmd = get_olecmd(command_id, &oleopt)) != IDM_UNKNOWN)
    {
        /* send the OLE command */
        icmd_->Exec(&CGID_MSHTML, olecmd, oleopt, 0, 0);

        /* handled */
        return TRUE;
    }

    /* inherit base class behavior for other commands */
    return CTadsWin::do_command(notify_code, command_id, ctl);
}

/*
 *   handle a non-client left click 
 */
int CHtmlSys_dbghelpwin::do_nc_leftbtn_down(
    int keys, int x, int y, int clicks, int hit_type)
{
    /* if it's a title-bar hit, track the context menu */
    if (hit_type == HTCAPTION)
    {
        /* 
         *   if focus isn't within our window, move focus here - this makes
         *   it easy to move focus back here from a docked window 
         */
        HWND f = GetFocus();
        if (f != handle_ && !IsChild(handle_, f))
            SetFocus(handle_);
    }

    /* proceed to the standard system handling */
    return FALSE;
}


/*
 *   handle a non-client right-click 
 */
int CHtmlSys_dbghelpwin::do_nc_rightbtn_down(
    int keys, int x, int y, int clicks, int hit_type)
{
    /* if it's a title-bar hit, track the context menu */
    if (hit_type == HTCAPTION)
    {
        POINT pt;

        /* convert the location to client coordinates */
        pt.x = x;
        pt.y = y;
        ScreenToClient(handle_, &pt);

        /* show the system menu as a context menu */
        track_system_context_menu(pt.x, pt.y);

        /* handled */
        return TRUE;
    }

    /* we didn't handle it */
    return FALSE;
}

/*
 *   move 
 */
void CHtmlSys_dbghelpwin::do_move(int x, int y)
{
    /* remember the change in the preferences after the move has completed */
    PostMessage(handle_, HTMLM_SAVE_WIN_POS, 0, 0);
}

/*
 *   resize 
 */
void CHtmlSys_dbghelpwin::do_resize(int mode, int x, int y)
{
    HWND active;
    int is_active, maxed;
    
    /* 
     *   If this is a "restore," and the active MDI window is currently
     *   maximized, don't resize the control.  When switching away from a
     *   maximized MDI child to a different child, Windows restores the
     *   outgoing child to its normal size.  If we switch back to the current
     *   window later, while still in MDI-maximized mode, we will have done
     *   two resizes on this window.  These resizes effectively cancel each
     *   other out, but the WebBrowser control tends to move around the
     *   scroll position on resize in a non-reversible way, such that a
     *   series of resizes that takes the window back to its original
     *   position still changes the scrolling position from where we started.
     *   This is undesirable in MDI-max mode, since merely switching away
     *   from this window for a moment and then switching back can cause the
     *   contents to scroll.  
     */
    active = (HWND)SendMessage(dbgmain_->get_parent_of_children(),
                               WM_MDIGETACTIVE, 0, (LPARAM)&maxed);
    is_active = (active == handle_ || IsChild(active, handle_));
    if (!maxed || is_active)
    {   
        /* resize the html control to match */
        if (iwb2_ != 0)
        {
            iwb2_->put_Width(x);
            iwb2_->put_Height(y);
        }
    }

    /* remember the change in the preferences after the move has completed */
    PostMessage(handle_, HTMLM_SAVE_WIN_POS, 0, 0);
}

/*
 *   handle a user message 
 */
int CHtmlSys_dbghelpwin::do_user_message(int msg, WPARAM wpar, LPARAM lpar)
{
    /* see what we have */
    switch (msg)
    {
    case HTMLM_SAVE_WIN_POS:
        /* do the deferred window position saving */
        store_winpos_prefs();
        return TRUE;
    }

    /* inherit the default handling */
    return CTadsWin::do_user_message(msg, wpar, lpar);
}

/*
 *   Store my position in the preferences.  This is invoked from our
 *   self-posted follow-up message, which we post during a size/move event
 *   for deferred processing after the triggering event has finished.  
 */
void CHtmlSys_dbghelpwin::store_winpos_prefs()
{
    RECT rc;
    CHtmlRect pos;
    HWND hwnd, mdihwnd;
    CHtmlDebugConfig *lc;

    /* get my frame position */
    hwnd = get_frame_pos(&rc);

    /* ignore this if we're not the active MDI window, or we're maximized */
    mdihwnd = dbgmain_->get_active_mdi_child();
    if ((mdihwnd != hwnd && !IsChild(mdihwnd, hwnd)) || is_win_maximized())
        return;

    /* convert to our local position structure */
    pos.set(rc.left, rc.top, rc.right, rc.bottom);

    /* store the position under our key */
    if ((lc = dbgmain_->get_local_config()) != 0)
        lc->setval("help win pos", 0, &pos);
}

/*
 *   handle notifications 
 */
int CHtmlSys_dbghelpwin::do_notify(
    int control_id, int notify_code, LPNMHDR nm)
{
    int new_focus;
    
    /* check the message */
    switch(notify_code)
    {
    case NM_SETFOCUS:
    case NM_KILLFOCUS:
        /* check which it is */
        new_focus = (notify_code == NM_SETFOCUS);

        /* notify myself of an artificial activation */
        do_parent_activate(new_focus);

        /* note the new status */
        has_focus_ = new_focus;

        /* inherit the default handling as well */
        break;

    default:
        /* no special handling */
        break;
    }

    /* inherit default handling */
    return CTadsWin::do_notify(control_id, notify_code, nm);
}

/* 
 *   handle non-client activation 
 */
int CHtmlSys_dbghelpwin::do_ncactivate(int flag)
{
    /* set active/inactive with the main window */
    if (dbgmain_ != 0)
        dbgmain_->notify_active_dbg_win(this, flag);

    /* inherit default handling */
    return CTadsWin::do_ncactivate(flag);
}

/*
 *   Receive notification of activation from parent 
 */
void CHtmlSys_dbghelpwin::do_parent_activate(int flag)
{
    /* set active/inactive with the main window */
    if (dbgmain_ != 0)
        dbgmain_->notify_active_dbg_win(this, flag);

    /* note the new status */
    has_focus_ = flag;

    /* inherit default handling */
    CTadsWin::do_parent_activate(flag);
}

/*
 *   gain focus 
 */
void CHtmlSys_dbghelpwin::do_setfocus(HWND prv)
{
    IDispatch *doc;
    
    /* inherit default */
    CTadsWin::do_setfocus(prv);

    /* set active/inactive with the main window */
    if (dbgmain_ != 0)
        dbgmain_->notify_active_dbg_win(this, TRUE);

    /* set focus on the html document */
    if (iwb2_ != 0
        && SUCCEEDED(iwb2_->get_Document(&doc)))
    {
        IHTMLDocument4 *doc4;
        
        /* get the IHtmlDocument4 interface from the document */
        if (SUCCEEDED(doc->QueryInterface(
            IID_IHTMLDocument4, (void **)&doc4)))
        {
            /* set focus on the document */
            doc4->focus();
            
            /* done with the doc4 */
            doc4->Release();
        }

        /* done with the document */
        doc->Release();
    }

    /* note the new status */
    has_focus_ = TRUE;
}

/*
 *   lose focus 
 */
void CHtmlSys_dbghelpwin::do_killfocus(HWND nxt)
{
    /* set active/inactive with the main window */
    if (dbgmain_ != 0)
        dbgmain_->notify_active_dbg_win(this, FALSE);

    /* note the new status */
    has_focus_ = FALSE;

    /* inherit default */
    CTadsWin::do_killfocus(nxt);
}

/*
 *   navigate to a URL 
 */
void CHtmlSys_dbghelpwin::go_to_url(const textchar_t *url)
{
    VARIANT v;

    /* if there's no browser control, we can't proceed */
    if (iwb2_ == 0)
        return;

    /* set up a Variant with the target URL */
    VariantInit(&v);
    v.vt = VT_BSTR;
    v.bstrVal = bstr_from_ansi(url);

    /* navigate to our page */
    iwb2_->Navigate2(&v, 0, 0, 0, 0);

    /* free the BSTR */
    VariantClear(&v);
}

/*
 *   handle timers 
 */
int CHtmlSys_dbghelpwin::do_timer(int timer_id)
{
    /* check for our focus timer */
    if (timer_id == focus_timer_)
    {
        int new_focus;
        
        /* check to see if focus is in this window or a child */
        HWND f = GetFocus();
        new_focus = (f == handle_ || IsChild(handle_, f));

        /* check to see if this is changed from last tim e*/
        if (new_focus != has_focus_)
        {
            if (new_focus)
                do_setfocus(0);
            else
                do_killfocus(0);
        }
    }

    /* inherit default */
    return CTadsWin::do_timer(timer_id);
}

